﻿/*
 * 以Button和Lamp說明DIP與interface的應用
 * 10/12/2008
 */

using System;

namespace ButtonAndLamp
{
    public interface SwitchableDevice
    {
        void TurnOn();
        void TurnOff();
    }
    public enum LampStatus
    {
        OFF = 0,
        ON = 1
    }
    public enum ButtonStatus
    {
        RELEASED = 0,
        PRESSED = 1
    }

    public class Lamp : SwitchableDevice 
    {
        private LampStatus status = LampStatus.OFF;
        public LampStatus Status
        {
            get { return status; }
        }
        public void TurnOn()
        {
            status = LampStatus.ON;
        }
        public void TurnOff()
        {
            status = LampStatus.OFF;
        }
    }


    public class Button
    {
        private ButtonStatus status = ButtonStatus.RELEASED;
        private SwitchableDevice device;
        public Button(SwitchableDevice device)
        {
            this.device = device;
        }
        public ButtonStatus Status
        {
            get { return status; }
        }
        public void Press()
        {
            if (status == ButtonStatus.RELEASED)
            {
                status = ButtonStatus.PRESSED;
                device.TurnOn();
            }
        }
        public void Release()
        {
            if (status == ButtonStatus.PRESSED)
            {
                status = ButtonStatus.RELEASED;
                device.TurnOff();
            }
        }

    }
    class Program
    {
        static void Main(string[] args)
        {
            Lamp lamp1 = new Lamp();
            Button button = new Button(lamp1);
            Random rand = new Random();
            for (int n = 0; n <= 100; ++n)
            {
                Console.Write("time n = " + n + "\t");
                if (rand.Next() % 2 == 1)
                {
                    if (button.Status == ButtonStatus.PRESSED)
                    {
                        button.Release();
                    }
                    else
                    {
                        button.Press();
                    }
                }
                if (lamp1.Status == LampStatus.OFF)
                {
                    Console.WriteLine("lamp1 is off");
                }
                else
                {
                    Console.WriteLine("lamp1 is on");
                }
                Console.WriteLine();
            }
        }
    }
}
